#if !defined(_ADATA_ADMXRC3_IOCTL_ADMXRC3_H)
#define _ADATA_ADMXRC3_IOCTL_ADMXRC3_H

/*
** File: ioctl_admxrc3.h  
** Project: ADMXRC3 API module library
** Purpose: Defines the IOCTL codes and structures for the ADMXRC3 API
**
** (C) Copyright Alpha Data 2009-2010
**
** This file expects the following definitions to exist in the namespace:
**
**   ADMXRC3_HAS_USER_EVENT_HANDLE    - 0 or 1
**   ADMXRC3_NEED_THUNK               - if building for a platform where thunking to 32 bits is required (e.g. x86_64)
**   user_event_handle_t              - if ADMXRC3_HAS_USER_EVENT_HANDLE is 1
**   user_event_handle_t_32           - if ADMXRC3_HAS_USER_EVENT_HANDLE is 1 and ADMXRC3_NEED_THUNK is defined
*/

#define ADMXRC3_IOCTLCODE_GETCARDINFO               (1)
#define ADMXRC3_IOCTLCODE_GETBANKINFO               (2)
#define ADMXRC3_IOCTLCODE_GETFPGAINFO               (3)
#define ADMXRC3_IOCTLCODE_GETDRIVERVERSION          (4)
#define ADMXRC3_IOCTLCODE_GETWINDOWINFO             (5)
#define ADMXRC3_IOCTLCODE_CONFIGURE                 (7)
#define ADMXRC3_IOCTLCODE_GETCLOCKFREQUENCY         (8)
#define ADMXRC3_IOCTLCODE_SETCLOCKFREQUENCY         (9)
#define ADMXRC3_IOCTLCODE_REGISTEREVENT             (10)
#define ADMXRC3_IOCTLCODE_UNREGISTEREVENT           (11)
#define ADMXRC3_IOCTLCODE_READ                      (12)
#define ADMXRC3_IOCTLCODE_WRITE                     (13)
#define ADMXRC3_IOCTLCODE_ERASEFLASH                (14)
#define ADMXRC3_IOCTLCODE_READFLASH                 (15)
#define ADMXRC3_IOCTLCODE_WRITEFLASH                (16)
#define ADMXRC3_IOCTLCODE_GETFLASHINFO              (17)
#define ADMXRC3_IOCTLCODE_GETFLASHBLOCKINFO         (18)
#define ADMXRC3_IOCTLCODE_LOCK                      (19)
#define ADMXRC3_IOCTLCODE_UNLOCK                    (20)
#define ADMXRC3_IOCTLCODE_READDMA                   (21)
#define ADMXRC3_IOCTLCODE_WRITEDMA                  (22)
#define ADMXRC3_IOCTLCODE_READDMALOCKED             (23)
#define ADMXRC3_IOCTLCODE_WRITEDMALOCKED            (24)
#define ADMXRC3_IOCTLCODE_READVPD                   (25)
#define ADMXRC3_IOCTLCODE_WRITEVPD                  (26)
#define ADMXRC3_IOCTLCODE_UNCONFIGURE               (27)
#define ADMXRC3_IOCTLCODE_NOTIFICATIONWAIT          (28)
#define ADMXRC3_IOCTLCODE_GETCOOPLEVEL              (29)
#define ADMXRC3_IOCTLCODE_SETCOOPLEVEL              (30)
#define ADMXRC3_IOCTLCODE_GETCARDINFOEX             (31)
#define ADMXRC3_IOCTLCODE_GETMODULEINFO             (32)
#define ADMXRC3_IOCTLCODE_GETSENSORINFO             (33)
#define ADMXRC3_IOCTLCODE_READSENSOR                (34)
#define ADMXRC3_IOCTLCODE_READSENSORNOTYPE          (35)

typedef union _IOCTLS_ADMXRC3_CONFIGURE {
  struct {
    uint32_t targetIndex;
    uint32_t flags;
    const void* pBuffer;
    size_t length;
  } in;
} IOCTLS_ADMXRC3_CONFIGURE;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_CONFIGURE {
  struct {
    uint32_t targetIndex;
    uint32_t flags;
    const_ptr32_t pBuffer;
    size32_t length;
  } in;
} IOCTLS32_ADMXRC3_CONFIGURE;
#endif

typedef union _IOCTLS_ADMXRC3_ERASEFLASH {
  struct {
    uint32_t flashIndex;
    uint32_t flags;
    uint64_t offset;
    uint64_t length;
  } in;
} IOCTLS_ADMXRC3_ERASEFLASH;

typedef union _IOCTLS_ADMXRC3_GETBANKINFO {
  struct {
    uint32_t bankIndex;
  } in;
  struct {
    uint64_t maximumFrequency;
    uint64_t minimumFrequency;
    uint64_t physicalSize;
    uint16_t physicalDataWidth;
    uint16_t physicalEccWidth;
    uint32_t typeMask;
    uint32_t connectivityMask;
    uint8_t bPresent;
  } out;
} IOCTLS_ADMXRC3_GETBANKINFO;

typedef union _IOCTLS_ADMXRC3_GETCARDINFO {
  struct {
    uint32_t model;
    uint32_t serialNumber;
    uint32_t numClockGen;
    uint32_t numDmaChannel;
    uint32_t numFlashBank;
    uint32_t numMemoryBank;
    uint32_t numTargetFpga;
    uint32_t numWindow;
    uint32_t memoryBankPresent;
  } out;
} IOCTLS_ADMXRC3_GETCARDINFO;

typedef union _IOCTLS_ADMXRC3_GETCARDINFOEX {
  struct {
    /* These members must be exactly as in IOCTLS_ADMXRC3_GETCARDINFO */
    uint32_t model;
    uint32_t serialNumber;
    uint32_t numClockGen;
    uint32_t numDmaChannel;
    uint32_t numFlashBank;
    uint32_t numMemoryBank;
    uint32_t numTargetFpga;
    uint32_t numWindow;
    uint32_t memoryBankPresent;
    /* Extra members not in IOCTLS_ADMXRC3_GETCARDINFO */
    uint32_t numSensor;
    uint32_t numModuleSite;
  } out;
} IOCTLS_ADMXRC3_GETCARDINFOEX;

typedef union _IOCTLS_ADMXRC3_GETCOOPLEVEL {
  struct {
    uint32_t cooperativeLevel;
  } out;
} IOCTLS_ADMXRC3_GETCOOPLEVEL;

typedef union _IOCTLS_ADMXRC3_GETCLOCKFREQUENCY {
  struct {
    uint32_t clockIndex;
  } in;
  struct {
    uint64_t frequency;
  } out;
} IOCTLS_ADMXRC3_GETCLOCKFREQUENCY;

typedef union _IOCTLS_ADMXRC3_GETDRIVERVERSION {
  struct {
    uint16_t major;
    uint16_t minor;
    uint16_t bugfix;
  } out;
} IOCTLS_ADMXRC3_GETDRIVERVERSION;

typedef union _IOCTLS_ADMXRC3_GETFLASHBLOCKINFO {
  struct {
    uint32_t flashIndex;
    uint32_t _pad1;
    uint64_t address;
  } in;
  struct {
    uint64_t address;
    uint64_t length;
    uint32_t flags;
    uint32_t _pad1;
  } out;
} IOCTLS_ADMXRC3_GETFLASHBLOCKINFO;

typedef union _IOCTLS_ADMXRC3_GETFLASHINFO {
  struct {
    uint32_t flashIndex;
  } in;
  struct {
    uint64_t size;
    uint64_t useableStart;
    uint64_t useableLength;
    uint16_t vendorId;
    uint16_t deviceId;
    uint32_t _pad1;
  } out;
} IOCTLS_ADMXRC3_GETFLASHINFO;

typedef union _IOCTLS_ADMXRC3_GETFPGAINFO {
  struct {
    uint32_t targetIndex;
  } in;
  struct {
    uint32_t deviceCode;
    uint32_t familyCode;
    uint32_t subfamilyCode;
    uint32_t packageCode;
    uint32_t flags;
    char speedGrade[8];
    char stepping[8];
    uint8_t bPresent;
  } out;
} IOCTLS_ADMXRC3_GETFPGAINFO;

typedef struct _IOCTLSUBS_ADMXRC3_GETMODULEINFO {
  char product[64];
  char partNumber[64];
  char manufacturer[64];
  char serialNumber[64];
  struct {
    int16_t intPart;
    uint16_t fracPart;
  } ioVoltage;
  uint32_t manufactureDate;
  uint32_t flags;
  uint32_t bPresent;
} IOCTLSUBS_ADMXRC3_GETMODULEINFO;

typedef union _IOCTLS_ADMXRC3_GETMODULEINFO {
  struct {
    uint32_t moduleIndex;
    IOCTLSUBS_ADMXRC3_GETMODULEINFO* pBuffer;
  } in;
} IOCTLS_ADMXRC3_GETMODULEINFO;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_GETMODULEINFO {
  struct {
    uint32_t moduleIndex;
    ptr32_t pBuffer;
  } in;
} IOCTLS32_ADMXRC3_GETMODULEINFO;
#endif

typedef union _IOCTLS_ADMXRC3_GETWINDOWINFO {
  struct {
    uint32_t windowIndex;
  } in;
  struct {
    uint64_t busSize;
    uint64_t busBase;
    uint64_t localSize;
    uint64_t localBase;
    uint64_t virtualSize;
  } out;
} IOCTLS_ADMXRC3_GETWINDOWINFO;

typedef union _IOCTLS_ADMXRC3_GETMMAPADDR {
  struct {
    uint32_t windowIndex;
    uint64_t offset;
    uint64_t length;
  } in;
  struct {
    uint64_t address;
    uint64_t length;
    uint32_t pageOffset;
  } out;
} IOCTLS_ADMXRC3_GETMMAPADDR;

typedef union _IOCTLS_ADMXRC3_LOCK {
  struct {
    const void* pBuffer;
    size_t length;
  } in;
  struct {
    uint32_t hBuffer;
  } out;
} IOCTLS_ADMXRC3_LOCK;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_LOCK {
  struct {
    ptr32_t pBuffer;
    size32_t length;
  } in;
  struct {
    uint32_t hBuffer;
  } out;
} IOCTLS32_ADMXRC3_LOCK;
#endif

typedef union _IOCTLS_ADMXRC3_NOTIFICATIONWAIT {
  struct {
    uint32_t notification;
  } in;
} IOCTLS_ADMXRC3_NOTIFICATIONWAIT;

typedef union _IOCTLS_ADMXRC3_READ {
  struct {
    uint32_t windowIndex;
    uint32_t flags;
    size_t offset;
    size_t length;
    void* pBuffer;
  } in;
} IOCTLS_ADMXRC3_READ;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_READ {
  struct {
    uint32_t windowIndex;
    uint32_t flags;
    size32_t offset;
    size32_t length;
    ptr32_t pBuffer;
  } in;
} IOCTLS32_ADMXRC3_READ;
#endif

typedef union _IOCTLS_ADMXRC3_READDMA {
  struct {
    uint32_t channel;
    uint32_t flags;
    void* pBuffer;
    size_t length;
    uint32_t local;
  } in;
} IOCTLS_ADMXRC3_READDMA;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_READDMA {
  struct {
    uint32_t channel;
    uint32_t flags;
    ptr32_t pBuffer;
    size32_t length;
    uint32_t local;
  } in;
} IOCTLS32_ADMXRC3_READDMA;
#endif

typedef union _IOCTLS_ADMXRC3_READDMALOCKED {
  struct {
    uint32_t channel;
    uint32_t flags;
    uint32_t hBuffer;
    size_t offset;
    size_t length;
    uint32_t local;
  } in;
} IOCTLS_ADMXRC3_READDMALOCKED;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_READDMALOCKED {
  struct {
    uint32_t channel;
    uint32_t flags;
    uint32_t hBuffer;
    size32_t offset;
    size32_t length;
    uint32_t local;
  } in;
} IOCTLS32_ADMXRC3_READDMALOCKED;
#endif

typedef union _IOCTLS_ADMXRC3_READFLASH {
  struct {
    uint32_t flashIndex;
    uint32_t flags;
    uint64_t offset;
    size_t length;
    void* pBuffer;
  } in;
} IOCTLS_ADMXRC3_READFLASH;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_READFLASH {
  struct {
    uint32_t flashIndex;
    uint32_t flags;
    uint64_t offset;
    size32_t length;
    const_ptr32_t pBuffer;
  } in;
} IOCTLS32_ADMXRC3_READFLASH;
#endif

typedef union _IOCTLS_ADMXRC3_READVPD {
  struct {
    uint32_t flags;
    size_t offset;
    size_t length;
    void* pBuffer;
  } in;
} IOCTLS_ADMXRC3_READVPD;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_READVPD {
  struct {
    uint32_t flags;
    size32_t offset;
    size32_t length;
    const_ptr32_t pBuffer;
  } in;
} IOCTLS32_ADMXRC3_READVPD;
#endif

#if ADMXRC3_HAS_USER_EVENT_HANDLE
typedef union _IOCTLS_ADMXRC3_REGISTEREVENT {
  struct {
    uint32_t            notification;
    user_event_handle_t hEvent;
  } in;
} IOCTLS_ADMXRC3_REGISTEREVENT;

# if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_REGISTEREVENT {
  struct {
    uint32_t               notification;
    user_event_handle_t_32 hEvent;
  } in;
} IOCTLS32_ADMXRC3_REGISTEREVENT;
# endif
#endif

typedef union _IOCTLS_ADMXRC3_SETCLOCKFREQUENCY {
  struct {
    uint32_t clockIndex;
    uint32_t flags;
    uint64_t frequency;
  } in;
  struct {
    uint64_t actualFrequency;
  } out;
} IOCTLS_ADMXRC3_SETCLOCKFREQUENCY;

typedef union _IOCTLS_ADMXRC3_SETCOOPLEVEL {
  struct {
    uint32_t cooperativeLevel;
  } in;
} IOCTLS_ADMXRC3_SETCOOPLEVEL;

typedef union _IOCTLS_ADMXRC3_UNCONFIGURE {
  struct {
    uint32_t targetIndex;
    uint32_t flags;
  } in;
} IOCTLS_ADMXRC3_UNCONFIGURE;

typedef union _IOCTLS_ADMXRC3_UNLOCK {
  struct {
    uint32_t hBuffer;
  } in;
} IOCTLS_ADMXRC3_UNLOCK;

#if ADMXRC3_HAS_USER_EVENT_HANDLE
typedef union _IOCTLS_ADMXRC3_UNREGISTEREVENT {
  struct {
    uint32_t            notification;
    user_event_handle_t hEvent;
  } in;
} IOCTLS_ADMXRC3_UNREGISTEREVENT;

# if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_UNREGISTEREVENT {
  struct {
    uint32_t               notification;
    user_event_handle_t_32 hEvent;
  } in;
} IOCTLS32_ADMXRC3_UNREGISTEREVENT;
# endif
#endif

typedef union _IOCTLS_ADMXRC3_WRITE {
  struct {
    uint32_t windowIndex;
    uint32_t flags;
    size_t offset;
    size_t length;
    const void* pBuffer;
  } in;
} IOCTLS_ADMXRC3_WRITE;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_WRITE {
  struct {
    uint32_t windowIndex;
    uint32_t flags;
    size32_t offset;
    size32_t length;
    const_ptr32_t pBuffer;
  } in;
} IOCTLS32_ADMXRC3_WRITE;
#endif

typedef union _IOCTLS_ADMXRC3_WRITEDMA {
  struct {
    uint32_t channel;
    uint32_t flags;
    const void* pBuffer;
    size_t length;
    uint32_t local;
  } in;
} IOCTLS_ADMXRC3_WRITEDMA;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_WRITEDMA {
  struct {
    uint32_t channel;
    uint32_t flags;
    const_ptr32_t pBuffer;
    size32_t length;
    uint32_t local;
  } in;
} IOCTLS32_ADMXRC3_WRITEDMA;
#endif

typedef union _IOCTLS_ADMXRC3_WRITEDMALOCKED {
  struct {
    uint32_t channel;
    uint32_t flags;
    uint32_t hBuffer;
    size_t offset;
    size_t length;
    uint32_t local;
  } in;
} IOCTLS_ADMXRC3_WRITEDMALOCKED;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_WRITEDMALOCKED {
  struct {
    uint32_t channel;
    uint32_t flags;
    uint32_t hBuffer;
    size32_t offset;
    size32_t length;
    uint32_t local;
  } in;
} IOCTLS32_ADMXRC3_WRITEDMALOCKED;
#endif

typedef union _IOCTLS_ADMXRC3_WRITEFLASH {
  struct {
    uint32_t flashIndex;
    uint32_t flags;
    uint64_t offset;
    size_t length;
    const void* pBuffer;
  } in;
} IOCTLS_ADMXRC3_WRITEFLASH;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_WRITEFLASH {
  struct {
    uint32_t flashIndex;
    uint32_t flags;
    uint64_t offset;
    size32_t length;
    const_ptr32_t pBuffer;
  } in;
} IOCTLS32_ADMXRC3_WRITEFLASH;
#endif

typedef union _IOCTLS_ADMXRC3_WRITEVPD {
  struct {
    uint32_t flags;
    size_t offset;
    size_t length;
    const void* pBuffer;
  } in;
} IOCTLS_ADMXRC3_WRITEVPD;

#if defined(ADMXRC3_NEED_THUNK)
typedef union _IOCTLS32_ADMXRC3_WRITEVPD {
  struct {
    uint32_t flags;
    size32_t offset;
    size32_t length;
    const_ptr32_t pBuffer;
  } in;
} IOCTLS32_ADMXRC3_WRITEVPD;
#endif

typedef union _IOCTLS_ADMXRC3_GETINFO {
  struct {
    uint32_t numSensor;
  } out;
} IOCTLS_ADMXRC3_GETINFO;

typedef union _IOCTLS_ADMXRC3_GETSENSORINFO {
  struct {
    uint32_t sensorIndex;
  } in;
  struct {
    char description[28];
    uint32_t capabilities;
    struct {
      int16_t intPart;
      uint16_t fracPart;
    } error;
    uint32_t datatype;
    uint32_t unit;
    int32_t exponent;
  } out;
} IOCTLS_ADMXRC3_GETSENSORINFO;

typedef union _IOCTLS_ADMXRC3_READSENSOR {
  struct {
    uint32_t sensorIndex;
    uint32_t datatype;
  } in;
  struct {
    union {
      struct {
        int16_t intPart;
        uint16_t fracPart;
      } doubleValue;
      uint32_t booleanValue;
      uint32_t uint32Value;
      int32_t int32Value;
    } value;
  } out;
} IOCTLS_ADMXRC3_READSENSOR;

typedef union _IOCTLS_ADMXRC3_READSENSORNOTYPE {
  struct {
    uint32_t sensorIndex;
  } in;
  struct {
    union {
      struct {
        int16_t intPart;
        uint16_t fracPart;
      } doubleValue;
      uint32_t booleanValue;
      uint32_t uint32Value;
      int32_t int32Value;
    } value;
    uint32_t datatype;
  } out;
} IOCTLS_ADMXRC3_READSENSORNOTYPE;

#endif
